﻿//
// MainPage.xaml.cpp
// Implementation of the MainPage class.
//

#include "pch.h"
#include "MainPage.xaml.h"

#define NUM_OSCILLATORS 6
#define NUM_COLUMNS     18  // could be 14, 18, or 22
#define TRANSPOSE       0   // could be set to 1 if using fewer buttons

using namespace ChromaticButtonKeyboard;

using namespace Platform;
using namespace Platform::Collections;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

MainPage::MainPage()
{
	InitializeComponent();

    // Create all the keys
    static int keyNums[5][4] = { 37, 40, 43, 46, 
                                 35, 38, 41, 44,
                                 36, 39, 42, 45,
                                 34, 37, 40, 43,
                                 35, 38, 41, 44 };

    static wchar_t * notes[] = { L"C", L"C#", L"D", L"D#", L"E", L"F", 
                                 L"F#", L"G", L"G#", L"A", L"A#", L"B" };

    Brush^ whiteBrush = ref new SolidColorBrush(Colors::White);
    Brush^ blackBrush = ref new SolidColorBrush(Colors::Black);
    Brush^ grayBrush = ref new SolidColorBrush(ColorHelper::FromArgb(255, 192, 192, 192));

    layoutCanvas->Width = 100 * NUM_COLUMNS;
    layoutCanvas->Height = 500;

    for (int row = 0; row < 5; row++)
    {
        bool isEvenRow = (row & 1) == 0;
        int cols = NUM_COLUMNS - (int)isEvenRow;

        for (int col = 0; col < cols; col++)
        {
            int keyNum = keyNums[row][col % 4] + 12 * (col / 4 + TRANSPOSE);
            wchar_t result[20];
            swprintf(result, 20, L"%s", notes[keyNum % 12]);

            Key^ key = ref new Key();
            key->Tag = row * 1000 + keyNum;
            key->Content = ref new String(result);
            key->Foreground = blackBrush;
            key->Background = wcslen(result) == 2 ? grayBrush : whiteBrush;
            key->Width = 90;
            key->Height = 90;
            key->FontSize = 48;
            key->IsPressedChanged += ref new EventHandler<bool>(this, &MainPage::OnKeyIsPressedChanged);

            Canvas::SetTop(key, row * 100 + 5);
            Canvas::SetLeft(key, col * 100 + (isEvenRow ? 55 : 5));
            layoutCanvas->Children->Append(key);
        }
    }

    // Create an IXAudio2 object
    HRESULT hr = XAudio2Create(&pXAudio2);

    XAUDIO2_DEBUG_CONFIGURATION debugConfig = {0};
    debugConfig.TraceMask = XAUDIO2_LOG_DETAIL | XAUDIO2_LOG_WARNINGS;
    pXAudio2->SetDebugConfiguration(&debugConfig);

    if (FAILED(hr))
        ref new COMException(hr, "XAudio2Create failure");

    // Create a mastering voice
    hr = pXAudio2->CreateMasteringVoice(&pMasteringVoice);

    if (FAILED(hr))
        ref new COMException(hr, "CreateMasteringVoice failure");

    // Create half a dozen oscillators
    for (int i = 0; i < NUM_OSCILLATORS; i++)
    {
        SawtoothOscillator* oscillator = new SawtoothOscillator(pXAudio2.Get());
        oscillator->SetAmplitude(0);
        availableOscillators.push_back(oscillator);
    }
}

void MainPage::OnKeyIsPressedChanged(Object^ sender, bool isPressed)
{
    Key^ key = dynamic_cast<Key^>(sender);
    int keyNum = (int)key->Tag;

    if (isPressed)
    {
        if (availableOscillators.size() > 0)
        {
            SawtoothOscillator* pOscillator = availableOscillators.back();
            availableOscillators.pop_back();

            double freq = 440 * pow(2, (keyNum % 1000 - 69) / 12.0);
            pOscillator->SetFrequency((float)freq);
            pOscillator->SetAmplitude(1.0f / NUM_OSCILLATORS);
            playingOscillators[keyNum] = pOscillator;
        }
    }
    else
    {
        SawtoothOscillator * pOscillator = playingOscillators[keyNum];

        if (pOscillator != nullptr)
        {
            pOscillator->SetAmplitude(0);
            availableOscillators.push_back(pOscillator);
            playingOscillators.erase(keyNum);
        }
    }
}
